/*===========================================================================
 *
 * File:	Esmarmor.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __ESMARMOR_H
#define __ESMARMOR_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "EsmItem3.h"
  #include "EsmSubAODT.h"
  #include "EsmSubByte.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Number of body part definitions allowed in an armor record */
  #define MWESM_ARMOR_MAXBODYPARTS 7

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CEsmArmor Definition
 *
 * Description
 *
 *=========================================================================*/
class CEsmArmor : public CEsmItem3 {
  DECLARE_SUBRECCREATE();


  /*---------- Begin Protected Class Members --------------------*/
protected:
  CEsmSubAODT*		m_pArmorData;	/* Reference to subrecords */


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CEsmArmor();
  //virtual ~CEsmArmor() { Destroy(); }
  virtual void Destroy (void);

	/* Compare two fields of the record */
  virtual int CompareFields (const int FieldID, CEsmRecord* pRecord);

  	/* Return a new record object */
  static CEsmRecord* Create (void);

  	/* Create a new, empty, record */
  virtual void CreateNew (CEsmFile* pFile);

  	/* Get a string representation of a particular field */
  virtual const TCHAR* GetFieldString (const int FieldID);

	/* Return a text representation of the item type */
  virtual const TCHAR* GetItemType (void) { return _T("Armor"); }

	/* Get class members */
  armordata_t*  GetArmorData	(void) { return (m_pArmorData == NULL ? NULL : m_pArmorData->GetArmorData() ); }
  const TCHAR*  GetArmorType	(void);
  int		GetArmorTypeID  (void) { return (m_pArmorData ? GetArmorData()->Type : -1); }
  virtual float	GetWeight	(void) { return (GetArmorData() == NULL ? 0 : GetArmorData()->Weight); }
  virtual long	GetValue	(void) { return (GetArmorData() == NULL ? 0 : GetArmorData()->Value); } 
  virtual long	GetEnchantPts   (void) { return (GetArmorData() == NULL ? 0 : GetArmorData()->EnchantPts); } 

	/* Used to save the various record elements */
  virtual void OnAddSubRecord (CEsmSubRecord* pSubRecord);

 };
/*===========================================================================
 *		End of Class CEsmArmor Definition
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Prototypes
 *
 *=========================================================================*/

	/* Convert an armor type to a string */
  const TCHAR* GetESMArmorType (const int ArmorType);

/*===========================================================================
 *		End of Function Prototypes
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Esmarmor.H
 *=========================================================================*/
