/*===========================================================================
 *
 * File:	EsmBodyPart.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "EsmBodyPart.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmBodyPart.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Local Type String Arrays
 *
 *=========================================================================*/
const TCHAR* l_BodyPartTypes[] = {
	_T("Skin"),
	_T("Clothing"),
	_T("Armor") };

const TCHAR* l_BodyParts[] = {
	_T("Head"),
	_T("Hair"),
	_T("Neck"),
	_T("Chest"),
	_T("Groin"),
	_T("Hand"),
	_T("Wrist"),
	_T("Forearm"),
	_T("Upperarm"),
	_T("Foot"),
	_T("Ankle"),
	_T("Knee"),
	_T("Upperleg"),
	_T("Clavicle"),
	_T("Tail") } ;
/*===========================================================================
 *		End of Local Type String Arrays
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Sub-Record Create Array
 *
 *=========================================================================*/
const esmsubreccreate_t CEsmBodyPart::s_SubRecCreate[] = {
	{ MWESM_SUBREC_NAME,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_FNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_MODL,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_BYDT,	CEsmSubBYDT::Create },
	{ NULL,			CEsmSubRecord::Create }	/* Must be last record */
 };
/*===========================================================================
 *		End of Sub-Record Create Array
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmBodyPart Constructor
 *
 *=========================================================================*/
CEsmBodyPart::CEsmBodyPart () {
  //DEFINE_FUNCTION("CEsmBodyPart::CEsmBodyPart()");
  m_pName     = NULL;
  m_pBodyData = NULL;
  m_pModel    = NULL;
 }
/*===========================================================================
 *		End of Class CEsmBodyPart Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmBodyPart Method - void Destroy (void);
 *
 * Description
 *
 *=========================================================================*/
void CEsmBodyPart::Destroy (void) {
  //DEFINE_FUNCTION("CEsmBodyPart::Destroy()");
  m_pName     = NULL;
  m_pBodyData = NULL;
  m_pModel    = NULL;

  CEsmRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CEsmBodyPart::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmBodyPart Method - int CompareFields (FieldID, pRecord);
 *
 * Compares the given field of the current record and the supplied record.
 * Returns a value which can be used for sorting the records..
 *
 *=========================================================================*/
int CEsmBodyPart::CompareFields (const int FieldID, CEsmRecord* pRecord) {
  CEsmBodyPart* pBody;

	/* Ensure the correct type */
  if (!pRecord->IsType(MWESM_REC_BODY)) return CEsmRecord::CompareFields(FieldID, pRecord);
  pBody = (CEsmBodyPart *) pRecord;

  switch (FieldID) {
    case ESM_FIELD_MODEL:
	return StringCompare(GetModel(), pBody->GetModel());
    case ESM_FIELD_NAME:
    case ESM_FIELD_RACE:
	return StringCompare(GetRace(), pBody->GetRace());
    case ESM_FIELD_FEMALE:
	return (int)IsFemale() - (int)pBody->IsFemale();
    case ESM_FIELD_PLAYABLE:
	return (int)IsPlayable() - (int)pBody->IsPlayable();
    case ESM_FIELD_TYPE:
	return StringCompare(GetBodyPartType(), pBody->GetBodyPartType());
    case ESM_FIELD_PART:
	return StringCompare(GetBodyPart(), pBody->GetBodyPart());
    default:
	return CEsmRecord::CompareFields(FieldID, pRecord); 
   }
 }
/*===========================================================================
 *		End of Class Method CEsmBodyPart::CompareFields()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmBodyPart Method - CEsmRecord* Create (void);
 *
 * Static class method to create a new record object.
 *
 *=========================================================================*/
CEsmRecord* CEsmBodyPart::Create (void) {
  DEFINE_FUNCTION("CEsmBodyPart::Create()");
  CEsmRecord* pRecord;

  CreatePointer(pRecord, CEsmBodyPart);
  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmBodyPart::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmBodyPart Method - void CreateNew (pFile);
 *
 * Creates a new, empty, record.
 *
 *=========================================================================*/
void CEsmBodyPart::CreateNew (CEsmFile* pFile) {

	/* Call the base class record first */
  CEsmRecord::CreateNew(pFile);

	/* Create the item sub-records */
  AllocateSubRecord(MWESM_SUBREC_FNAM);
  AllocateSubRecord(MWESM_SUBREC_MODL);
  AllocateSubRecord(MWESM_SUBREC_BYDT);
  m_pBodyData->CreateNew();
 }
/*===========================================================================
 *		End of Class Method CEsmBodyPart::CreateNew()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TCHAR* CEsmBodyPart Method - const GetFieldString (FieldID);
 *
 * Returns a string representation of the given field. Always returns
 * a valid string.
 *
 *=========================================================================*/
const TCHAR* CEsmBodyPart::GetFieldString (const int FieldID) {

  switch (FieldID) {
    case ESM_FIELD_MODEL:
	return (GetModel());
    case ESM_FIELD_NAME:
    case ESM_FIELD_RACE:
	return (GetRace());
    case ESM_FIELD_FEMALE:
	return (BOOLTOYESNO(IsFemale()));
    case ESM_FIELD_TYPE:
	return (GetBodyPartType());
    case ESM_FIELD_PART:
	return (GetBodyPart());
    case ESM_FIELD_PLAYABLE:
	return (BOOLTOYESNO(IsPlayable()));
    default:
	return CEsmRecord::GetFieldString(FieldID);
   }
  
 }
/*===========================================================================
 *		End of Class Method TCHAR* CEsmBodyPart::GetFieldString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmBodyPart Event - void OnAddSubRecord (pSubRecord);
 *
 *=========================================================================*/
void CEsmBodyPart::OnAddSubRecord (CEsmSubRecord* pSubRecord) {

  if (pSubRecord->IsType(MWESM_SUBREC_BYDT))
    m_pBodyData = (CEsmSubBYDT *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_FNAM))
    m_pName = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_MODL))
    m_pModel = (CEsmSubNameFix *) pSubRecord;
  else
    CEsmRecord::OnAddSubRecord(pSubRecord);

 }
/*===========================================================================
 *		End of Class Event CEsmArmor::CEsmBodyPart()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - const TCHAR* GetESMBodyPart (Part);
 *
 * Converts a part value into a string.
 *
 *=========================================================================*/
const TCHAR* GetESMBodyPart (const int Part) {
  if (Part < MWESM_PART_MIN || Part > MWESM_PART_MAX) return _T("Unknown");

  return l_BodyParts[Part];
 }
/*===========================================================================
 *		End of Function TCHAR* GetESMBodyPart()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - const TCHAR* GetESMBodyPartType (PartType);
 *
 * Converts a part type into a string.
 *
 *=========================================================================*/
const TCHAR* GetESMBodyPartType (const int PartType) {
  if (PartType < MWESM_PARTTYPE_MIN || PartType > MWESM_PARTTYPE_MAX) return _T("Unknown");

  return l_BodyPartTypes[PartType];
 }
/*===========================================================================
 *		End of Function TCHAR* GetESMBodyPartType()
 *=========================================================================*/
