/*===========================================================================
 *
 * File:	EsmBook.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __ESMBOOK_H
#define __ESMBOOK_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "EsmItem3.h"
  #include "EsmSubBKDT.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/
/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CEsmBook Definition
 *
 * Description
 *
 *=========================================================================*/
class CEsmBook : public CEsmItem3 {
  DECLARE_SUBRECCREATE();


  /*---------- Begin Protected Class Members --------------------*/
protected:
  CEsmSubBKDT*		m_pBookData;	/* Reference to subrecords */
  CEsmSubName*		m_pText;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CEsmBook();
  //virtual ~CEsmBook() { Destroy(); }
  virtual void Destroy (void);

	/* Compare two fields of the record */
  virtual int CompareFields (const int FieldID, CEsmRecord* pRecord);

  	/* Return a new record object */
  static CEsmRecord* Create (void);

  	/* Create a new, empty, record */
  virtual void CreateNew (CEsmFile* pFile);

  	/* Get a string representation of a particular field */
  virtual const TCHAR* GetFieldString (const int FieldID);

	/* Return a text representation of the item type */
  virtual const TCHAR* GetItemType (void) { return _T("Book"); }

	/* Get class members */
  bookdata_t*   GetBookData	(void) { return (m_pBookData == NULL ? NULL : m_pBookData->GetBookData() ); }
  const TCHAR*  GetBookText	(void) { return (m_pText ? m_pText->GetName() : _T("")); }
  virtual float	GetWeight	(void) { return (GetBookData() == NULL ? 0 : GetBookData()->Weight); }
  virtual long	GetValue	(void) { return (GetBookData() == NULL ? 0 : GetBookData()->Value); } 
  virtual long	GetEnchantPts   (void) { return (GetBookData() == NULL ? 0 : GetBookData()->EnchantPts); } 
  int		GetSkillID	(void) { return (m_pBookData ? GetBookData()->SkillID : -1); }
  bool		IsScroll	(void) { return (m_pBookData ? (GetBookData()->Scroll != 0): false); }

	/* Used to save the various record elements */
  virtual void OnAddSubRecord (CEsmSubRecord* pSubRecord);

	/* Set class members */
  void SetScroll     (const bool  Flag)   { if (m_pBookData) GetBookData()->Scroll     = (int) Flag; }
  void SetSkillID    (const int   Skill)  { if (m_pBookData) GetBookData()->SkillID    = Skill; }
  void SetEnchantPts (const int   Value)  { if (m_pBookData) GetBookData()->EnchantPts = Value; }
  void SetValue      (const int   Value)  { if (m_pBookData) GetBookData()->Value      = Value; }
  void SetWeight     (const float Weight) { if (m_pBookData) GetBookData()->Weight     = Weight; }
  void SetBookText   (const TCHAR* pText) { if (m_pText) m_pText->SetName(pText); }

 };
/*===========================================================================
 *		End of Class CEsmBook Definition
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Prototypes
 *
 *=========================================================================*/

	/* Convert an armor type to a string */
  const TCHAR* GetESMArmorType (const int ArmorType);

/*===========================================================================
 *		End of Function Prototypes
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Esmarmor.H
 *=========================================================================*/
