/*===========================================================================
 *
 * File:	EsmClass.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "EsmClass.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmClass.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Sub-Record Create Array
 *
 *=========================================================================*/
const esmsubreccreate_t CEsmClass::s_SubRecCreate[] = {
	{ MWESM_SUBREC_NAME,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_FNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_DESC,	CEsmSubName::Create },
	{ MWESM_SUBREC_CLDT,	CEsmSubCLDT::Create },
	{ NULL,			CEsmSubRecord::Create }	/* Must be last record */
 };
/*===========================================================================
 *		End of Sub-Record Create Array
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmClass Constructor
 *
 *=========================================================================*/
CEsmClass::CEsmClass () {
  //DEFINE_FUNCTION("CEsmClass::CEsmClass()");
  m_pName	 = NULL;
  m_pData	 = NULL;
  m_pDescription = NULL;
 }
/*===========================================================================
 *		End of Class CEsmClass Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmClass Method - void Destroy (void);
 *
 *=========================================================================*/
void CEsmClass::Destroy (void) {
  //DEFINE_FUNCTION("CEsmClass::Destroy()");
  m_pName	 = NULL;
  m_pData	 = NULL;
  m_pDescription = NULL;
  
  CEsmRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CEsmClass::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmClass Method - int CompareFields (FieldID, pRecord);
 *
 * Compares the given field of the current record and the supplied record.
 * Returns a value which can be used for sorting the records..
 *
 *=========================================================================*/
int CEsmClass::CompareFields (const int FieldID, CEsmRecord* pRecord) {
  CEsmClass* pClass;

	/* Ensure the correct type */
  if (!pRecord->IsType(MWESM_REC_CLAS)) return CEsmRecord::CompareFields(FieldID, pRecord);
  pClass = (CEsmClass *) pRecord;

  switch (FieldID) {
    case ESM_FIELD_NAME:
	return StringCompare(GetName(), pClass->GetName(), false);
    case ESM_FIELD_PLAYABLE:
	return (int)IsPlayable() - (int)pClass->IsPlayable();
    default:
	return CEsmRecord::CompareFields(FieldID, pRecord); 
   }

 }
/*===========================================================================
 *		End of Class Method CEsmClass::CompareFields()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmClass Method - CEsmRecord* Create (void);
 *
 * Static class method to create a new record object.
 *
 *=========================================================================*/
CEsmRecord* CEsmClass::Create (void) {
  DEFINE_FUNCTION("CEsmClass::Create()");
  CEsmRecord* pRecord;

  CreatePointer(pRecord, CEsmClass);
  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmClass::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmClass Method - void CreateNew (pFile);
 *
 * Creates a new, empty, record.
 *
 *=========================================================================*/
void CEsmClass::CreateNew (CEsmFile* pFile) {

	/* Call the base class record first */
  CEsmRecord::CreateNew(pFile);

	/* Create the item sub-records */
  AllocateSubRecord(MWESM_SUBREC_FNAM);
  AllocateSubRecord(MWESM_SUBREC_DESC);
  AllocateSubRecord(MWESM_SUBREC_CLDT);
  m_pData->CreateNew();
 }
/*===========================================================================
 *		End of Class Method CEsmClass::CreateNew()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TCHAR* CEsmClass Method - const GetFieldString (FieldID);
 *
 * Returns a string representation of the given field. Always returns
 * a valid string.
 *
 *=========================================================================*/
const TCHAR* CEsmClass::GetFieldString (const int FieldID) {
  static TCHAR s_Buffer[32];

  switch (FieldID) {
    case ESM_FIELD_NAME:
	return GetName();
    case ESM_FIELD_PLAYABLE:
	return BOOLTOYESNO(IsPlayable());
    default:
	return CEsmRecord::GetFieldString(FieldID);
   }
  
 }
/*===========================================================================
 *		End of Class Method TCHAR* CEsmClass::GetFieldString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmClass Event - void OnAddSubRecord (pSubRecord);
 *
 *=========================================================================*/
void CEsmClass::OnAddSubRecord (CEsmSubRecord* pSubRecord) {

  if (pSubRecord->IsType(MWESM_SUBREC_FNAM))
    m_pName = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_DESC))
    m_pDescription = (CEsmSubName *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_CLDT))
    m_pData = (CEsmSubCLDT *) pSubRecord;
  else
    CEsmRecord::OnAddSubRecord(pSubRecord);

 }
/*===========================================================================
 *		End of Class Event CEsmArmor::CEsmClass()
 *=========================================================================*/


