/*===========================================================================
 *
 * File:	EsmCreature.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __ESMCREATURE_H
#define __ESMCREATURE_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "EsmItem1.h"
  #include "EsmSubCRDT.h"
  #include "EsmSubNPCO.h"
  #include "EsmSubAIDT.h"
  #include "EsmSubLong.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Creature flags */
  #define MWESM_CREAFLAG_BIPED		0x0001
  #define MWESM_CREAFLAG_RESPAWN	0x0002
  #define MWESM_CREAFLAG_WEAPSHIELD	0x0004
  #define MWESM_CREAFLAG_NONE		0x0008
  #define MWESM_CREAFLAG_SWIMS		0x0010
  #define MWESM_CREAFLAG_FLIES 		0x0020
  #define MWESM_CREAFLAG_WALKS		0x0040
  #define MWESM_CREAFLAG_DEFAULT	0x0048
  #define MWESM_CREAFLAG_ESSENTIAL	0x0080
  #define MWESM_CREAFLAG_WHITEBLOOD	0x0400
  #define MWESM_CREAFLAG_GOLDBLOOD	0x0800
  #define MWESM_CREAFLAG_BLOODMASK	0x0F00

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Prototypes
 *
 *=========================================================================*/
  const TCHAR* GetESMCreatureType (const int CreaType);
/*===========================================================================
 *		End of Function Prototypes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CEsmCreature Definition
 *
 * Description
 *
 *=========================================================================*/
class CEsmCreature : public CEsmItem1 {
  DECLARE_SUBRECCREATE();
 
  /*---------- Begin Protected Class Members --------------------*/
protected:
  CEsmSubCRDT*		m_pCreatureData;
  CEsmSubLong*		m_pFlag;
  CEsmSubAIDT*		m_pAIData;
  CEsmSubNameFix*	m_pSound;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CEsmCreature();
  //virtual ~CEsmCreature() { Destroy(); }
  virtual void Destroy (void);

  	/* Compare two fields of the record */
  virtual int CompareFields (const int FieldID, CEsmRecord* pRecord);

  	/* Return a new record object */
  static CEsmRecord* Create (void);
 
  	/* Create a new, empty, record */
  virtual void CreateNew (CEsmFile* pFile);

  	/* Get a string representation of a particular field */
  virtual const TCHAR* GetFieldString (const int FieldID);

  	/* Return a text representation of the item type */
  virtual const TCHAR* GetItemType (void) { return _T("Creature"); }

	/* Get class members */
  aidata_t*	  GetAIData	  (void) { return (m_pAIData ? m_pAIData->GetAIData() : NULL); }
  creaturedata_t* GetCreatureData (void) { return (m_pCreatureData == NULL ? NULL : m_pCreatureData->GetCreatureData()); }
  const TCHAR*    GetCreaType     (void) { return (GetCreatureData() == NULL ? _T("Unknown") : GetESMCreatureType(GetCreatureData()->Type)); }
  const TCHAR*    GetMoveType     (void);
  const TCHAR*    GetSound        (void) { return (m_pSound ? m_pSound->GetName() : _T("")); }
  long		  GetFlag	  (void) { return (m_pFlag == NULL ? 0 : m_pFlag->GetValue()); }

	/* Get creature data */
  long GetLevel		(void) { return (m_pCreatureData ? GetCreatureData()->Level : 0); }
  long GetStrength	(void) { return (m_pCreatureData ? GetCreatureData()->Strength : 0); }
  long GetIntelligence	(void) { return (m_pCreatureData ? GetCreatureData()->Intelligence : 0); }
  long GetWillPower	(void) { return (m_pCreatureData ? GetCreatureData()->Willpower : 0); }
  long GetAgility	(void) { return (m_pCreatureData ? GetCreatureData()->Agility : 0); }
  long GetSpeed		(void) { return (m_pCreatureData ? GetCreatureData()->Speed : 0); }
  long GetEndurance	(void) { return (m_pCreatureData ? GetCreatureData()->Endurance : 0); }
  long GetPersonality	(void) { return (m_pCreatureData ? GetCreatureData()->Personality : 0); }
  long GetLuck		(void) { return (m_pCreatureData ? GetCreatureData()->Luck : 0); }
  long GetHealth	(void) { return (m_pCreatureData ? GetCreatureData()->Health : 0); }
  long GetSpellPts	(void) { return (m_pCreatureData ? GetCreatureData()->SpellPts : 0); }
  long GetFatigue	(void) { return (m_pCreatureData ? GetCreatureData()->Fatigue : 0); }
  long GetSoul		(void) { return (m_pCreatureData ? GetCreatureData()->Soul : 0); }
  long GetCombat	(void) { return (m_pCreatureData ? GetCreatureData()->Combat : 0); }
  long GetMagic		(void) { return (m_pCreatureData ? GetCreatureData()->Magic : 0); }
  long GetStealth	(void) { return (m_pCreatureData ? GetCreatureData()->Stealth : 0); }
  long GetAttackMin1	(void) { return (m_pCreatureData ? GetCreatureData()->AttackMin1 : 0); }
  long GetAttackMin2	(void) { return (m_pCreatureData ? GetCreatureData()->AttackMin2 : 0); }
  long GetAttackMin3	(void) { return (m_pCreatureData ? GetCreatureData()->AttackMin3 : 0); }
  long GetAttackMax1	(void) { return (m_pCreatureData ? GetCreatureData()->AttackMax1 : 0); }
  long GetAttackMax2	(void) { return (m_pCreatureData ? GetCreatureData()->AttackMax2 : 0); }
  long GetAttackMax3	(void) { return (m_pCreatureData ? GetCreatureData()->AttackMax3 : 0); }

	/* Get creature flags */
  bool	IsEssential  (void) { return ((GetFlag() & MWESM_CREAFLAG_ESSENTIAL) != 0); }
  bool	IsRespawn    (void) { return ((GetFlag() & MWESM_CREAFLAG_RESPAWN) != 0); }
  bool	IsBiped      (void) { return ((GetFlag() & MWESM_CREAFLAG_BIPED) != 0); }
  bool	IsWeapShield (void) { return ((GetFlag() & MWESM_CREAFLAG_WEAPSHIELD) != 0); }
  bool	IsSwims      (void) { return ((GetFlag() & MWESM_CREAFLAG_SWIMS) != 0); }
  bool	IsFlies      (void) { return ((GetFlag() & MWESM_CREAFLAG_FLIES) != 0); }
  bool	IsWalks      (void) { return ((GetFlag() & MWESM_CREAFLAG_WALKS) != 0); }
  bool	IsNone       (void) { return ((GetFlag() & MWESM_CREAFLAG_NONE) != 0); }
  
	/* Used to save the various record elements */
  virtual void OnAddSubRecord (CEsmSubRecord* pSubRecord);

	/* Set class members */
  void SetFlag (const long Value) { if (m_pFlag) m_pFlag->SetValue(Value); }
  void SetSound (const TCHAR* pString);

 };
/*===========================================================================
 *		End of Class CEsmCreature Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File EsmActivator.H
 *=========================================================================*/
