/*===========================================================================
 *
 * File:	EsmGlobal.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "EsmGlobal.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmGlobal.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Sub-Record Create Array
 *
 *=========================================================================*/
const esmsubreccreate_t CEsmGlobal::s_SubRecCreate[] = {
	{ MWESM_SUBREC_NAME,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_FNAM,	CEsmSubByte::Create },
	{ MWESM_SUBREC_FLTV,	CEsmSubFloat::Create },
	{ NULL,			CEsmSubRecord::Create }	/* Must be last record */
 };
/*===========================================================================
 *		End of Sub-Record Create Array
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmGlobal Constructor
 *
 *=========================================================================*/
CEsmGlobal::CEsmGlobal () {
  //DEFINE_FUNCTION("CEsmGlobal::CEsmGlobal()");
  m_pTypeData   = NULL;
  m_pFloatData  = NULL;
 }
/*===========================================================================
 *		End of Class CEsmGlobal Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmGlobal Method - void Destroy (void);
 *
 *=========================================================================*/
void CEsmGlobal::Destroy (void) {
  //DEFINE_FUNCTION("CEsmGlobal::Destroy()");
  m_pTypeData   = NULL;
  m_pFloatData  = NULL;
  
  CEsmRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CEsmGlobal::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmGlobal Method - int CompareFields (FieldID, pRecord);
 *
 * Compares the given field of the current record and the supplied record.
 * Returns a value which can be used for sorting the records..
 *
 *=========================================================================*/
int CEsmGlobal::CompareFields (const int FieldID, CEsmRecord* pRecord) {
  CEsmGlobal* pGlobal;

	/* Ensure the correct type */
  if (!pRecord->IsType(MWESM_REC_GLOB)) return CEsmRecord::CompareFields(FieldID, pRecord);
  pGlobal = (CEsmGlobal *) pRecord;

  switch (FieldID) {
    case ESM_FIELD_VALUE:
	return (int)(GetValue()*1000 - pGlobal->GetValue()*1000);
    case ESM_FIELD_TYPE:
	return (int)(GetType() - pGlobal->GetType());
    default:
	return CEsmRecord::CompareFields(FieldID, pRecord); 
   }

 }
/*===========================================================================
 *		End of Class Method CEsmGlobal::CompareFields()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmGlobal Method - CEsmRecord* Create (void);
 *
 * Static class method to create a new record object.
 *
 *=========================================================================*/
CEsmRecord* CEsmGlobal::Create (void) {
  DEFINE_FUNCTION("CEsmGlobal::Create()");
  CEsmRecord* pRecord;

  CreatePointer(pRecord, CEsmGlobal);
  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmGlobal::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmGlobal Method - void CreateNew (pFile);
 *
 * Creates a new, empty, record.
 *
 *=========================================================================*/
void CEsmGlobal::CreateNew (CEsmFile* pFile) {

	/* Call the base class record first */
  CEsmRecord::CreateNew(pFile);

	/* Create the item sub-records */
  AllocateSubRecord(MWESM_SUBREC_FNAM);
  AllocateSubRecord(MWESM_SUBREC_FLTV);
  m_pTypeData->CreateNew();
  m_pFloatData->CreateNew();
  SetType(MWESM_GLOBAL_SHORT);
 }
/*===========================================================================
 *		End of Class Method CEsmGlobal::CreateNew()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TCHAR* CEsmGlobal Method - const GetFieldString (FieldID);
 *
 * Returns a string representation of the given field. Always returns
 * a valid string.
 *
 *=========================================================================*/
const TCHAR* CEsmGlobal::GetFieldString (const int FieldID) {
  static TCHAR s_Buffer[32];

  switch (FieldID) {
    case ESM_FIELD_VALUE:
	if (GetType() == MWESM_GLOBAL_FLOAT)
	  snprintf (s_Buffer, 31, _T("%g"), GetValue());
	else
	  snprintf (s_Buffer, 31, _T("%d"), (int) GetValue());

	return (s_Buffer);
    case ESM_FIELD_TYPE:
	return GetTypeString();
    default:
	return CEsmRecord::GetFieldString(FieldID);
   }
  
 }
/*===========================================================================
 *		End of Class Method TCHAR* CEsmGlobal::GetFieldString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TCHAR* CEsmGlobal Method - const GetTypeString (void);
 *
 *=========================================================================*/
const TCHAR* CEsmGlobal::GetTypeString (void) {

  switch (GetType()) {
    case MWESM_GLOBAL_FLOAT:
	return _T("float");
    case MWESM_GLOBAL_LONG:
	return _T("long");
    case MWESM_GLOBAL_SHORT:
	return _T("short");
    default:
	return _T("unknown");
   }

 }
/*===========================================================================
 *		End of Class Method TCHAR* CEsmGlobal::GetTypeString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmGlobal Event - void OnAddSubRecord (pSubRecord);
 *
 *=========================================================================*/
void CEsmGlobal::OnAddSubRecord (CEsmSubRecord* pSubRecord) {

  if (pSubRecord->IsType(MWESM_SUBREC_FNAM))
    m_pTypeData = (CEsmSubByte *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_FLTV))
    m_pFloatData = (CEsmSubFloat *) pSubRecord;
  else
    CEsmRecord::OnAddSubRecord(pSubRecord);

 }
/*===========================================================================
 *		End of Class Event CEsmArmor::CEsmGlobal()
 *=========================================================================*/


