/*===========================================================================
 *
 * File:	EsmIngrediant.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __ESMINGREDIANT_H
#define __ESMINGREDIANT_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "EsmItem2.h"
  #include "EsmSubENAM.h"
  #include "EsmSubIRDT.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

  #define MWESM_INGRE_NUMENCHANTS 4

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CEsmIngrediant Definition
 *
 * Description
 *
 *=========================================================================*/
class CEsmIngrediant : public CEsmItem2 {
  DECLARE_SUBRECCREATE();

  /*---------- Begin Protected Class Members --------------------*/
protected:
  CEsmSubIRDT*	m_pIngreData;		/* Reference to subrecords */


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CEsmIngrediant();
  //virtual ~CEsmIngrediant() { Destroy(); }
  virtual void Destroy (void);

	/* Compare two fields of the record */
  virtual int CompareFields (const int FieldID, CEsmRecord* pRecord);

  	/* Return a new record object */
  static CEsmRecord* Create (void);

  	/* Create a new, empty, record */
  virtual void CreateNew (CEsmFile* pFile);

  	/* Get a string representation of a particular field */
  virtual const TCHAR* GetFieldString (const int FieldID);

  	/* Return a text representation of the item type */
  virtual const TCHAR* GetItemType (void) { return _T("Ingrediant"); }

	/* Get class members */
  ingredata_t*   GetIngreData	 (void) { return (m_pIngreData ? m_pIngreData->GetIngreData() : NULL); }
  virtual float	 GetWeight	 (void) { return (m_pIngreData ? GetIngreData()->Weight : 0); }
  virtual long	 GetValue	 (void) { return (m_pIngreData ? GetIngreData()->Value  : 0); } 

	/* Access the effect/skill/attribute IDs */
  long GetEffectID    (const int Index) { if (!IsValidIndex(Index)) return (-1); return (m_pIngreData ? GetIngreData()->EffectID[Index] : -1); }
  long GetSkillID     (const int Index) { if (!IsValidIndex(Index)) return (-1); return (m_pIngreData ? GetIngreData()->SkillID[Index] : -1); }
  long GetAttributeID (const int Index) { if (!IsValidIndex(Index)) return (-1); return (m_pIngreData ? GetIngreData()->AttributeID[Index] : -1); }
  bool IsValidIndex   (const int Index) { return (Index >= 0 && Index < MWESM_INGRE_NUMEFFECTS); }
  const TCHAR* GetEffect    (const int Index) { return GetESMEffect(GetEffectID(Index)); }
  const TCHAR* GetSkill     (const int Index) { return GetESMSkill(GetEffectID(Index)); }
  const TCHAR* GetAttribute (const int Index) { return GetESMAttribute(GetEffectID(Index)); }

	/* Used to save the various record elements */
  virtual void OnAddSubRecord (CEsmSubRecord* pSubRecord);

 };
/*===========================================================================
 *		End of Class CEsmIngrediant Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File EsmIngrediant.H
 *=========================================================================*/
