/*===========================================================================
 *
 * File:	EsmLight.H
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/
#ifndef __ESMLIGHT_H
#define __ESMLIGHT_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "EsmItem2.h"
  #include "EsmSubLHDT.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Prototypes
 *
 *=========================================================================*/
  const TCHAR* GetESMLightFlicker (const int Flags);
/*===========================================================================
 *		End of Function Prototypes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CEsmLight Definition
 *
 * Description
 *
 *=========================================================================*/
class CEsmLight : public CEsmItem2 {
  DECLARE_SUBRECCREATE();

  /*---------- Begin Protected Class Members --------------------*/
protected:
  CEsmSubLHDT*		m_pLightData;	/* Reference to subrecords */
  CEsmSubNameFix*	m_pSound;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CEsmLight();
  //virtual ~CEsmLight() { Destroy(); }
  virtual void Destroy (void);

  	/* Compare two fields of the record */
  virtual int CompareFields (const int FieldID, CEsmRecord* pRecord);

  	/* Return a new record object */
  static CEsmRecord* Create (void);

  	/* Create a new, empty, record */
  virtual void CreateNew (CEsmFile* pFile);

	/* Return a text representation of the item type */
  virtual const TCHAR* GetItemType (void) { return _T("Light"); }

  	/* Get class members */
  lightdata_t*  GetLightData	(void) { return (m_pLightData == NULL ? NULL : m_pLightData->GetLightData()); }
  const TCHAR*  GetSound	(void) { return (m_pSound ? m_pSound->GetName() : _T("")); }
  virtual float	GetWeight	(void) { return (GetLightData() == NULL ? 0 : GetLightData()->Weight); }
  virtual long	GetValue	(void) { return (GetLightData() == NULL ? 0 : GetLightData()->Value); } 
  long		GetTime		(void) { return (m_pLightData ? GetLightData()->Time     : 0); }
  long		GetRadius	(void) { return (m_pLightData ? GetLightData()->Radius   : 0); }
  long		GetColor	(void) { return (m_pLightData ? GetLightData()->ColorRef : 0); }
  long		GetFlags	(void) { return (m_pLightData ? GetLightData()->Flags : 0); }
  byte		GetRed		(void) { return (m_pLightData ? GetLightData()->Red   : 0); }
  byte		GetGreen	(void) { return (m_pLightData ? GetLightData()->Green : 0); }
  byte		GetBlue		(void) { return (m_pLightData ? GetLightData()->Blue  : 0); }
  bool		IsDynamic	(void) { return (GetFlags() & MWESM_LIGHTFLAG_DYNAMIC) != 0; }
  bool		IsCanCarry	(void) { return (GetFlags() & MWESM_LIGHTFLAG_CANCARRY) != 0; }
  bool		IsNegative	(void) { return (GetFlags() & MWESM_LIGHTFLAG_NEGATIVE) != 0; }
  bool		IsFlicker	(void) { return (GetFlags() & MWESM_LIGHTFLAG_FLICKER) != 0; }
  bool		IsFire		(void) { return (GetFlags() & MWESM_LIGHTFLAG_ISFIRE) != 0; }
  bool		IsDefaultOff	(void) { return (GetFlags() & MWESM_LIGHTFLAG_DEFAULTOFF) != 0; }
  bool		IsFlickerSlow	(void) { return (GetFlags() & MWESM_LIGHTFLAG_FLICKERSLOW) != 0; }
  bool		IsPulse		(void) { return (GetFlags() & MWESM_LIGHTFLAG_PULSE) != 0; }
  bool		IsPulseSlow	(void) { return (GetFlags() & MWESM_LIGHTFLAG_PULSESLOW) != 0; }

  	/* Get a string representation of a particular field */
  virtual const TCHAR* GetFieldString (const int FieldID);

  	/* Used to save the various record elements */
  virtual void OnAddSubRecord (CEsmSubRecord* pSubRecord);

	/* Set class members */
  void SetIcon  (const TCHAR* pString);
  void SetName  (const TCHAR* pString);
  void SetSound (const TCHAR* pString);

 };
/*===========================================================================
 *		End of Class CEsmArmor Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Esmarmor.H
 *=========================================================================*/
