/*===========================================================================
 *
 * File:	EsmNpc.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "EsmNpc.h"
#include "EsmInfo.h"
#include "EsmSubPos6.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmNpc.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Sub-Record Create Array
 *
 *=========================================================================*/
const esmsubreccreate_t CEsmNpc::s_SubRecCreate[] = {
	{ MWESM_SUBREC_NAME,	CEsmSubName::Create },
	{ MWESM_SUBREC_FNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_MODL,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_SCRI,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_ANAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_BNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_CNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_KNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_RNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_NPDT,	CEsmSubNPDTL::Create },
	{ MWESM_SUBREC_FLAG,	CEsmSubLong::Create },
	{ MWESM_SUBREC_NPCO,	CEsmSubNPCO::Create },
	{ MWESM_SUBREC_NPCS,	CEsmSubNPCS::Create },
	{ MWESM_SUBREC_AIDT,	CEsmSubAIDT::Create },
	{ MWESM_SUBREC_AI_A,	CEsmSubAI_A::Create },
	{ MWESM_SUBREC_AI_E,	CEsmSubAI_E::Create },
	{ MWESM_SUBREC_AI_E,	CEsmSubAI_F::Create },
	{ MWESM_SUBREC_AI_T,	CEsmSubAI_T::Create },
	{ MWESM_SUBREC_AI_W,	CEsmSubAI_W::Create },
	{ MWESM_SUBREC_CNDT,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_DODT,	CEsmSubPos6::Create },
	{ MWESM_SUBREC_DNAM,	CEsmSubName::Create },
	{ MWESM_SUBREC_XSCL,	CEsmSubFloat::Create },
	{ NULL,			CEsmSubRecord::Create }	/* Must be last record */
 };
/*===========================================================================
 *		End of Sub-Record Create Array
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Constructor
 *
 *=========================================================================*/
CEsmNpc::CEsmNpc () {
  //DEFINE_FUNCTION("CEsmNpc::CEsmNpc()");
  m_pNpcShortData = NULL;
  m_pNpcLongData  = NULL;
  m_pFlag	  = NULL;
  m_pRace	  = NULL;
  m_pAnimation    = NULL;
  m_pHeadModel    = NULL;
  m_pHairModel    = NULL;
  m_pAIData       = NULL;
  m_pClass        = NULL;
  m_pFaction      = NULL;
 }
/*===========================================================================
 *		End of Class CEsmNpc Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - void Destroy (void);
 *
 * Description
 *
 *=========================================================================*/
void CEsmNpc::Destroy (void) {
  //DEFINE_FUNCTION("CEsmNpc::Destroy()");

	/* Clear the references */
  m_pNpcShortData = NULL;
  m_pNpcLongData  = NULL;
  m_pFlag	  = NULL;
  m_pRace	  = NULL;
  m_pAnimation    = NULL;
  m_pHeadModel    = NULL;
  m_pHairModel    = NULL;
  m_pAIData       = NULL;
  m_pClass        = NULL;
  m_pFaction      = NULL;

	/* Call the base class method */
  CEsmItem1::Destroy();
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmRecord Method - CEsmNpc* AllocNewSubRecord (pType);
 *
 * Allocates a new subrecord of the given type.
 *
 *=========================================================================*/
CEsmSubRecord* CEsmNpc::AllocNewSubRecord (const TCHAR* pType, const long RecordSize) {
  CEsmSubRecord* pSubRecord;

	/* Check for special case */
  if (TSTRICMP(pType, MWESM_SUBREC_NPDT) != 0) return CEsmItem1::AllocNewSubRecord(pType, RecordSize);

	/* Create and initialize the new record */
  if (RecordSize == sizeof(npclongdata_t)) 
    pSubRecord = CEsmSubNPDTL::Create();
  else if (RecordSize == sizeof(npclongdata_t)) 
    pSubRecord = CEsmSubNPDTS::Create();
  else 
    return CEsmItem1::AllocNewSubRecord(pType, RecordSize);

  pSubRecord->SetType(pType);
  pSubRecord->SetRecordSize(RecordSize);

  return (pSubRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::AllocNewSubRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - int CompareFields (FieldID, pRecord);
 *
 * Compares the given field of the current record and the supplied record.
 * Returns a value which can be used for sorting the records..
 *
 *=========================================================================*/
int CEsmNpc::CompareFields (const int FieldID, CEsmRecord* pRecord) {
  //DEFINE_FUNCTION("CEsmNpc::CompareFields()");
  CEsmNpc* pNpc;

	/* Ensure the correct type */
  if (!pRecord->IsType(MWESM_REC_NPC_)) return CEsmItem1::CompareFields(FieldID, pRecord);
  pNpc = (CEsmNpc *) pRecord;

  switch (FieldID) {
    case ESM_FIELD_FEMALE:
	return (int)IsFemale() - (int)pNpc->IsFemale();
    case ESM_FIELD_AUTOCALC:
	return (int)IsAutoCalc() - (int)pNpc->IsAutoCalc();
    case ESM_FIELD_ESSENTIAL:
	return (int)IsEssential() - (int)pNpc->IsEssential();
    case ESM_FIELD_RESPAWN:
	return (int)IsRespawn() - (int)pNpc->IsRespawn();
    case ESM_FIELD_LEVEL:
	return GetLevel() - pNpc->GetLevel();
    case ESM_FIELD_CLASS:
        return StringCompare(GetClass(), pNpc->GetClass(), true);
    case ESM_FIELD_RACE:
        return StringCompare(GetRace(), pNpc->GetRace(), true);
    case ESM_FIELD_ANIMATION:
        return StringCompare(GetAnimation(), pNpc->GetAnimation(), true);
    case ESM_FIELD_FACTION:
        return StringCompare(GetFaction(), pNpc->GetFaction(), true);
    case ESM_FIELD_RANK:
	return GetRank() - pNpc->GetRank();
    default:	/* Call the base class method */
	return CEsmItem1::CompareFields(FieldID, pRecord); 
   }
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::CompareFields()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - CEsmRecord* Create (void);
 *
 * Static class method to create a new record object.
 *
 *=========================================================================*/
CEsmRecord* CEsmNpc::Create (void) {
  DEFINE_FUNCTION("CEsmNpc::Create()");
  CEsmRecord* pRecord;

  CreatePointer(pRecord, CEsmNpc);
  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmArmor::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - void CreateNew (pFile);
 *
 * Creates a new, empty, record.
 *
 *=========================================================================*/
void CEsmNpc::CreateNew (CEsmFile* pFile) {

	/* Call the base class record first */
  CEsmItem1::CreateNew(pFile);
  DeleteSubRecords(MWESM_SUBREC_MODL);
  m_pAnimation = NULL;

	/* Create the item sub-records */
  AllocateSubRecord(MWESM_SUBREC_NPDT, sizeof(npcshortdata_t));
  AllocateSubRecord(MWESM_SUBREC_FLAG);
  AllocateSubRecord(MWESM_SUBREC_RNAM);
  AllocateSubRecord(MWESM_SUBREC_ANAM);
  AllocateSubRecord(MWESM_SUBREC_BNAM);
  AllocateSubRecord(MWESM_SUBREC_CNAM);
  AllocateSubRecord(MWESM_SUBREC_KNAM);
  AllocateSubRecord(MWESM_SUBREC_AIDT);
  m_pNpcShortData->CreateNew();
  m_pFlag->CreateNew();
  m_pFlag->SetValue(MWESM_NPCFLAG_DEFAULT);
  m_pAIData->CreateNew();
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::CreateNew()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - void CreateNPCData (IsShort);
 *
 *=========================================================================*/
void CEsmNpc::CreateNPCData (const bool IsShort) { 
  
	/* Ignore if the correct type is already created */
  if (IsShort  && m_pNpcShortData != NULL) return;
  if (!IsShort && m_pNpcLongData  != NULL) return;

	/* Delete the current data */
  DeleteSubRecords(MWESM_SUBREC_NPDT);
  m_pNpcShortData = NULL;
  m_pNpcLongData  = NULL;

	/* Create new data */
  if (IsShort) {
    AllocateSubRecord(MWESM_SUBREC_NPDT, sizeof(npcshortdata_t));
    m_pNpcShortData->CreateNew();
   }
  else {
    AllocateSubRecord(MWESM_SUBREC_NPDT, sizeof(npclongdata_t));
    m_pNpcLongData->CreateNew();
   }

 }
/*===========================================================================
 *		End of Class Method CEsmNpc::CreateNPCData()
 *=========================================================================*/
 

/*===========================================================================
 *
 * Class CEsmNpc Method - bool DoesInfoApply (pInfo);
 *
 *=========================================================================*/
bool CEsmNpc::DoesInfoApply (CEsmInfo* pInfo) {
  const TCHAR* pString;
  infodata_t*  pInfoData = pInfo->GetInfoData();
  
	/* Check ID */
  pString = pInfo->GetActor();
  if (*pString != NULL_CHAR && TSTRNICMP(pString, GetID(), MWESM_ID_MAXSIZE) != 0) return (false);

	/* Check gender */
  if (pInfoData->Gender != MWESM_INFOGENDER_NONE) {
    if (IsFemale()  && pInfoData->Gender == MWESM_INFOGENDER_MALE)   return (false);
    if (!IsFemale() && pInfoData->Gender == MWESM_INFOGENDER_FEMALE) return (false);
   }

	/* Check race */
  pString = pInfo->GetRace();
  if (*pString != NULL_CHAR && TSTRNICMP(pString, GetRace(), MWESM_ID_MAXSIZE) != 0) return (false);

  	/* Check class */
  pString = pInfo->GetClass();
  if (*pString != NULL_CHAR && TSTRNICMP(pString, GetClass(), MWESM_ID_MAXSIZE) != 0) return (false);

    	/* Check faction */
  pString = pInfo->GetFaction();
  if (*pString != NULL_CHAR && TSTRNICMP(pString, GetFaction(), MWESM_ID_MAXSIZE) != 0) return (false);

    	/* Check rank */
  if (pInfoData->Rank >= 0 && pInfoData->Rank > GetRank()) return (false);

	/* Check cell? */

  return (true);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::DoesInfoApply()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TCHAR* CEsmNpc Method - const GetFieldString (FieldID);
 *
 * Returns a string representation of the given field. Always returns
 * a valid string.
 *
 *=========================================================================*/
const TCHAR* CEsmNpc::GetFieldString (const int FieldID) {
  //DEFINE_FUNCTION("CEsmNpc::GetFieldString()");
  static TCHAR s_Buffer[32];

  switch (FieldID) {
    case ESM_FIELD_FEMALE:
	return BOOLTOYESNO(IsFemale());
    case ESM_FIELD_AUTOCALC:
	return BOOLTOYESNO(IsAutoCalc());
    case ESM_FIELD_ESSENTIAL:
	return BOOLTOYESNO(IsEssential());
    case ESM_FIELD_RESPAWN:
	return BOOLTOYESNO(IsRespawn());
    case ESM_FIELD_LEVEL:
	snprintf (s_Buffer, 31, _T("%ld"), GetLevel());
	return (s_Buffer);
    case ESM_FIELD_CLASS:
        return GetClass();
    case ESM_FIELD_RACE:
        return GetRace();
    case ESM_FIELD_ANIMATION:
        return GetAnimation();
    case ESM_FIELD_FACTION:
        return GetFaction();
    case ESM_FIELD_RANK:
        snprintf (s_Buffer, 31, _T("%ld"), GetRank());
	return (s_Buffer);
    default:	/* Call the base class record */
	return CEsmItem1::GetFieldString(FieldID);
   }
  
 }
/*===========================================================================
 *		End of Class Method TCHAR* CEsmNpc::GetFieldString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetDisposition (void);
 *
 *=========================================================================*/
long CEsmNpc::GetDisposition (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Disposition;
  else if (m_pNpcShortData != NULL)
    return m_pNpcShortData->GetNPCData()->Disposition;

  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetDisposition()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetReputation (void);
 *
 *=========================================================================*/
long CEsmNpc::GetReputation (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Reputation;
  else if (m_pNpcShortData != NULL)
    return 0;

  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetReputation()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetFactionID (void);
 *
 *=========================================================================*/
long CEsmNpc::GetFactionID (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->FactionID;
  else if (m_pNpcShortData != NULL)
    return m_pNpcShortData->GetNPCData()->FactionID;

  return (-1);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetFactionID()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetGold (void);
 *
 *=========================================================================*/
long CEsmNpc::GetGold (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Gold;
  else if (m_pNpcShortData != NULL)
    return m_pNpcShortData->GetNPCData()->Gold;

  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetGold()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetLevel (void);
 *
 *=========================================================================*/
long CEsmNpc::GetLevel (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Level;
  else if (m_pNpcShortData != NULL)
    return m_pNpcShortData->GetNPCData()->Level;

  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetLevel()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetHealth (void);
 *
 *=========================================================================*/
long CEsmNpc::GetHealth (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Health;
  
  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetHealth()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetMagic (void);
 *
 *=========================================================================*/
long CEsmNpc::GetMagic (void) {
  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->SpellPts;
  
  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetMagic()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetFatigue (void);
 *
 *=========================================================================*/
long CEsmNpc::GetFatigue (void) {
  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Fatigue;
  
  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetFatigue()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - long GetRank (void);
 *
 *=========================================================================*/
long CEsmNpc::GetRank (void) {

  if (m_pNpcLongData != NULL) 
    return m_pNpcLongData->GetNPCData()->Rank;
  else if (m_pNpcShortData != NULL)
    return m_pNpcShortData->GetNPCData()->Rank;

  return (0);
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::GetRank()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Event - void OnAddSubRecord (pSubRecord);
 *
 *=========================================================================*/
void CEsmNpc::OnAddSubRecord (CEsmSubRecord* pSubRecord) {

	/* There are two different NPDT record sizes */
  if (pSubRecord->IsType(MWESM_SUBREC_NPDT)) {
    if (pSubRecord->GetRecordSize() == sizeof(npcshortdata_t))
      m_pNpcShortData = (CEsmSubNPDTS *) pSubRecord;
    else if (pSubRecord->GetRecordSize() == sizeof(npclongdata_t))
      m_pNpcLongData  = (CEsmSubNPDTL *) pSubRecord;
   }
  else if (pSubRecord->IsType(MWESM_SUBREC_FLAG))
    m_pFlag = (CEsmSubLong *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_RNAM))
    m_pRace = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_MODL))
    m_pAnimation = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_BNAM))
    m_pHeadModel = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_CNAM))
    m_pClass = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_KNAM))
    m_pHairModel = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_ANAM))
    m_pFaction = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_AIDT))
    m_pAIData = (CEsmSubAIDT *) pSubRecord;
  else
    CEsmItem1::OnAddSubRecord(pSubRecord);
  
 }
/*===========================================================================
 *		End of Class Event CEsmNpc::OnAddSubRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmNpc Method - void SetAnimation (pString);
 *
 *=========================================================================*/
void CEsmNpc::SetAnimation (const TCHAR* pString) {
  DEFINE_FUNCTION("CEsmNpc::SetAnimation()");

  if (pString == NULL || *pString == NULL_CHAR) {
    if (m_pAnimation != NULL) DeleteSubRecords(MWESM_SUBREC_MODL);
    m_pAnimation = NULL;
   }
  else {
    if (m_pAnimation == NULL) AllocateSubRecord(MWESM_SUBREC_MODL);
    ASSERT(m_pAnimation != NULL);
    m_pAnimation->SetName(pString);
   }
 }

void CEsmNpc::SetDisposition (const int Value) {
  if (m_pNpcLongData != NULL)
    m_pNpcLongData->GetNPCData()->Disposition = Value;
  else if (m_pNpcShortData != NULL)
    m_pNpcShortData->GetNPCData()->Disposition = Value;
 }


void CEsmNpc::SetLevel (const int Value) {
  if (m_pNpcLongData != NULL)
    m_pNpcLongData->GetNPCData()->Level = Value;
  else if (m_pNpcShortData != NULL)
    m_pNpcShortData->GetNPCData()->Level = Value;
 }

void CEsmNpc::SetRank (const int Value) {
  if (m_pNpcLongData != NULL)
    m_pNpcLongData->GetNPCData()->Rank = Value;
  else if (m_pNpcShortData != NULL)
    m_pNpcShortData->GetNPCData()->Rank = Value;
 }

void CEsmNpc::SetGold (const long Value) {
  if (m_pNpcLongData != NULL)
    m_pNpcLongData->GetNPCData()->Gold = Value;
  else if (m_pNpcShortData != NULL)
    m_pNpcShortData->GetNPCData()->Gold = Value;
 }
/*===========================================================================
 *		End of Class Method CEsmNpc::SetAnimation()
 *=========================================================================*/


