/*===========================================================================
 *
 * File:	EsmSoundGen.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "EsmSoundGen.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmSoundGen.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Sub-Record Create Array
 *
 *=========================================================================*/
const esmsubreccreate_t CEsmSoundGen::s_SubRecCreate[] = {
	{ MWESM_SUBREC_NAME,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_SNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_CNAM,	CEsmSubNameFix::Create },
	{ MWESM_SUBREC_DATA,	CEsmSubLong::Create },
	{ NULL,			CEsmSubRecord::Create }	/* Must be last record */
 };
/*===========================================================================
 *		End of Sub-Record Create Array
 *=========================================================================*/


/*===========================================================================
 *
 * Begin String Arrays
 *
 *=========================================================================*/
const TCHAR* l_SoundGenTypes[] = {
	_T("Left Foot"),
	_T("Right Foot"),
	_T("Swim Left"),
	_T("Swim Right"),
	_T("Moan"),
	_T("Roar"),
	_T("Scream"),
	_T("Land"),
 };
/*===========================================================================
 *		End of String Arrays
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Constructor
 *
 *=========================================================================*/
CEsmSoundGen::CEsmSoundGen () {
  //DEFINE_FUNCTION("CEsmSoundGen::CEsmSoundGen()");
  m_pSoundName	  = NULL;
  m_pTypeData	  = NULL;
  m_pCreatureName = NULL;
 }
/*===========================================================================
 *		End of Class CEsmSoundGen Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Method - void Destroy (void);
 *
 *=========================================================================*/
void CEsmSoundGen::Destroy (void) {
  //DEFINE_FUNCTION("CEsmSoundGen::Destroy()");
  m_pSoundName	  = NULL;
  m_pTypeData	  = NULL;
  m_pCreatureName = NULL;
  
  CEsmRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CEsmSoundGen::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Method - int CompareFields (FieldID, pRecord);
 *
 * Compares the given field of the current record and the supplied record.
 * Returns a value which can be used for sorting the records..
 *
 *=========================================================================*/
int CEsmSoundGen::CompareFields (const int FieldID, CEsmRecord* pRecord) {
  CEsmSoundGen* pSoundGen;

	/* Ensure the correct type */
  if (!pRecord->IsType(MWESM_REC_SNDG)) return CEsmRecord::CompareFields(FieldID, pRecord);
  pSoundGen = (CEsmSoundGen *) pRecord;

  switch (FieldID) {
    case ESM_FIELD_CREATURE:
	return StringCompare(GetCreature(), pSoundGen->GetCreature(), false);
    case ESM_FIELD_TYPE:
	return GetSoundType() - pSoundGen->GetSoundType();
    case ESM_FIELD_SOUND:
	return StringCompare(GetSound(), pSoundGen->GetSound(), false);
    default:
	return CEsmRecord::CompareFields(FieldID, pRecord); 
   }

 }
/*===========================================================================
 *		End of Class Method CEsmSoundGen::CompareFields()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Method - CEsmRecord* Create (void);
 *
 * Static class method to create a new record object.
 *
 *=========================================================================*/
CEsmRecord* CEsmSoundGen::Create (void) {
  DEFINE_FUNCTION("CEsmSoundGen::Create()");
  CEsmRecord* pRecord;

  CreatePointer(pRecord, CEsmSoundGen);
  return (pRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmSoundGen::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Method - void CreateNew (pFile);
 *
 * Creates a new, empty, record.
 *
 *=========================================================================*/
void CEsmSoundGen::CreateNew (CEsmFile* pFile) {

	/* Call the base class record first */
  CEsmRecord::CreateNew(pFile);

	/* Create the item sub-records */
  AllocateSubRecord(MWESM_SUBREC_SNAM);
  AllocateSubRecord(MWESM_SUBREC_DATA);
  m_pTypeData->CreateNew();
  
 }
/*===========================================================================
 *		End of Class Method CEsmSoundGen::CreateNew()
 *=========================================================================*/


/*===========================================================================
 *
 * Class TCHAR* CEsmSoundGen Method - const GetFieldString (FieldID);
 *
 * Returns a string representation of the given field. Always returns
 * a valid string.
 *
 *=========================================================================*/
const TCHAR* CEsmSoundGen::GetFieldString (const int FieldID) {

  switch (FieldID) {
    case ESM_FIELD_TYPE:
	return GetTypeString();
    case ESM_FIELD_SOUND:
	return GetSound();
    case ESM_FIELD_CREATURE:
	return GetCreature();
    default:
	return CEsmRecord::GetFieldString(FieldID);
   }
  
 }
/*===========================================================================
 *		End of Class Method TCHAR* CEsmSoundGen::GetFieldString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Event - void OnAddSubRecord (pSubRecord);
 *
 *=========================================================================*/
void CEsmSoundGen::OnAddSubRecord (CEsmSubRecord* pSubRecord) {

  if (pSubRecord->IsType(MWESM_SUBREC_SNAM))
    m_pSoundName = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_CNAM))
    m_pCreatureName = (CEsmSubNameFix *) pSubRecord;
  else if (pSubRecord->IsType(MWESM_SUBREC_DATA))
    m_pTypeData = (CEsmSubLong *) pSubRecord;
  else
    CEsmRecord::OnAddSubRecord(pSubRecord);

 }
/*===========================================================================
 *		End of Class Event CEsmSoundGen::CEsmSoundGen()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSoundGen Method - void SetCreature (pString);
 *
 *=========================================================================*/
void CEsmSoundGen::SetCreature (const TCHAR* pString) {

	/* Delete the creature record */
  if (pString == NULL || *pString == NULL_CHAR) {
    if (m_pCreatureName != NULL) DeleteSubRecords(MWESM_SUBREC_CNAM);
    m_pCreatureName = NULL;
   }
	/* Set the creature record, create if required */
  else if (m_pCreatureName == NULL) {
    AllocateSubRecord(MWESM_SUBREC_CNAM);
    m_pCreatureName->SetName(pString); 
   }
  else {
    m_pCreatureName->SetName(pString);
   }

 }
/*===========================================================================
 *		End of Class Method CEsmSoundGen::SetCreature()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - const TCHAR* GetESMSoundGenType (Type);
 *
 *=========================================================================*/
const TCHAR* GetESMSoundGenType (const int Type) {
  if (Type < MWESM_SNDGEN_MIN || Type > MWESM_SNDGEN_MAX) return _T("Unknown");
  return l_SoundGenTypes[Type];
 }
/*===========================================================================
 *		End of Function TCHAR* GetESMSoundGenType()
 *=========================================================================*/
