/*===========================================================================
 *
 * File:	EsmSubCellRef.CPP
 * Author:	Dave Humphrey (uesp@m0use.net)
 * Created On:	February 3, 2003
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "EsmSubCellRef.h"
#include "EsmSubNameFix.h"
#include "EsmSubLong.h"
#include "EsmCell.h"


/*===========================================================================
 *
 * Begin Local Definitions
 *
 *=========================================================================*/
  DEFINE_FILE("EsmSubCellRef.cpp");
/*===========================================================================
 *		End of Local Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Constructor
 *
 *=========================================================================*/
CEsmSubCellRef::CEsmSubCellRef () : m_SubRecArray(0) {
  //DEFINE_FUNCTION("CEsmSubCellRef::CEsmSubCellRef()");
  m_IsActive = false;
  m_pRecInfo = NULL;
  m_pCell    = NULL;
  m_UserData = 0;
 }
/*===========================================================================
 *		End of Class CEsmSubCellRef Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - void Destroy (void);
 *
 *=========================================================================*/
void CEsmSubCellRef::Destroy (void) {
  //DEFINE_FUNCTION("CEsmSubCellRef::Destroy()");
  ClearSubRecArray();

  m_IsActive = false;
  m_pRecInfo = NULL;
  m_pCell    = NULL;

  CEsmSubRecord::Destroy();
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - void ClearSubRecArray (void);
 *
 * Deletes all record in the cellref array.
 *
 *=========================================================================*/
void CEsmSubCellRef::ClearSubRecArray (void) {
  DEFINE_FUNCTION("CEsmSubCellRef::ClearSubRecArray()");
  CEsmSubRecord* pSubRec;
  int		 Index;

  for (Index = 0; Index < m_SubRecArray.GetSize(); Index++) {
    pSubRec = m_SubRecArray.GetAt(Index);
    DestroyPointer(pSubRec);
   }

  m_SubRecArray.RemoveAll();
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::ClearSubRecArray()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - void Copy (pSubRecord);
 *
 *=========================================================================*/
void CEsmSubCellRef::Copy (CEsmSubRecord* pSubRecord) {
  CEsmSubRecord*  pSubRec;
  CEsmSubRecord*  pNewSubRec;
  CEsmSubCellRef* pCellRef;
  int		  Index;

	/* Clear the current contents */
  Destroy();

	/* Only copy records of the same type */
  if (!pSubRecord->IsType(MWESM_SUBREC_CREF)) return;
  m_Type.SetType(pSubRecord->GetType());
  pCellRef = (CEsmSubCellRef *) pSubRecord;
  m_pRecInfo = pCellRef->GetRecInfo();
  m_IsActive = pCellRef->IsActive();
  m_pCell    = pCellRef->GetCell();

	/* Copy all sub-records */
  for (Index = 0; Index < pCellRef->GetNumSubRecs(); Index++) {
    pSubRec = pCellRef->GetSubRec(Index);
    pNewSubRec = m_pCell->AllocNewSubRecord(pSubRec->GetType(), pSubRec->GetRecordSize());
    pNewSubRec->Copy(pSubRec);
    m_SubRecArray.Add(pNewSubRec);
   } 

 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::Copy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - CEsmSubRecord* Create (void);
 *
 *=========================================================================*/
CEsmSubRecord* CEsmSubCellRef::Create (void) {
  CEsmSubRecord* pSubRecord;
  CreatePointerL(pSubRecord, CEsmSubCellRef);
  return (pSubRecord);
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::Create()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - void CreateNew (void);
 *
 *=========================================================================*/
void CEsmSubCellRef::CreateNew (void) { 
  CEsmSubRecord::CreateNew();
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::CreateNew()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - CEsmSubRecord* FindSubRecord (pType);
 *
 * Finds the sub-record of the given type.  Cell-references should contain
 * only one of each type.  Returns NULL if the sub-record is not found.
 *
 *=========================================================================*/
CEsmSubRecord* CEsmSubCellRef::FindSubRecord (const TCHAR* pType) {
  CEsmSubRecord* pSubRec;
  int		 Index;

  for (Index = 0; Index < m_SubRecArray.GetSize(); Index++) {
    pSubRec = m_SubRecArray.GetAt(Index);
    if (pSubRec->IsType(pType)) return (pSubRec);
   }

  return (NULL);
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::FindSubRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - bool Find (FindData);
 *
 * Returns true if the given text existing in the sub-record data.  Default
 * implementation does a binary check of the sub-record data (not case
 * sensitive).  Stops searching on the first match.
 *
 *=========================================================================*/
bool CEsmSubCellRef::Find (esmfind_t& FindData) {
  CEsmSubRecord* pSubRec;
  bool		 Result;
  int		 Index;

	/* Search all sub-records */
  for (Index = 0; Index < m_SubRecArray.GetSize(); Index++) {
    pSubRec = m_SubRecArray.GetAt(Index);
    Result = pSubRec->Find(FindData);
    if (Result) return (true);
   }

	/* No match */
  return (false);
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::Find()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - bool IsSame (pCellRef);
 *
 * Returns true if the given input record is the same object reference as
 * the current object.
 *
 *=========================================================================*/
bool CEsmSubCellRef::IsSame (CEsmSubCellRef* pCellRef) {
  CEsmSubNameFix*  pName1;
  CEsmSubNameFix*  pName2;
  CEsmSubLong*	   pIndex1;
  CEsmSubLong*     pIndex2;

	/* Check the object names */
  pName1 = (CEsmSubNameFix *) FindSubRecord(MWESM_SUBREC_NAME);
  pName2 = (CEsmSubNameFix *) pCellRef->FindSubRecord(MWESM_SUBREC_NAME);
  if (pName1 == NULL || pName2 == NULL) return (false);
  //SystemLog.Printf ("Compare '%s' and '%s'", pName1->GetName(), pName2->GetName());
  if (TSTRICMP(pName1->GetName(), pName2->GetName()) != 0) return (false);

	/* Check the object indices */
  pIndex1 = (CEsmSubLong *) FindSubRecord(MWESM_SUBREC_FRMR);
  pIndex2 = (CEsmSubLong *) pCellRef->FindSubRecord(MWESM_SUBREC_FRMR);
  if (pIndex1 == NULL || pIndex2 == NULL) return (false);
  //SystemLog.Printf ("\t\tCompare '%ld' and '%ld'", pIndex1->GetValue(), pIndex2->GetValue());

  if ((pIndex1->GetValue() & MWESM_CELLREF_INDEXMASK) == (pIndex2->GetValue() & MWESM_CELLREF_INDEXMASK)) return (true);
  
  return (false);
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::IsSame()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CEsmSubCellRef Method - bool Write (File);
 *
 * Outputs all the sub-records contained within the cell reference.
 *
 *=========================================================================*/
bool CEsmSubCellRef::Write (CGenFile& File) {
  CEsmSubRecord* pSubRec;
  bool		 Result;
  int		 Index;

  for (Index = 0; Index < m_SubRecArray.GetSize(); Index++) {
    pSubRec = m_SubRecArray.GetAt(Index);
    Result = pSubRec->Write(File);
    if (!Result) return (false);
   }

  return (true); 
 }
/*===========================================================================
 *		End of Class Method CEsmSubCellRef::Write()
 *=========================================================================*/
